@@ -507,6 +507,31 @@ ORBit_iinterfaces_from_tree (IDL_tree                        tree,
 	return retval;
 }
 
+static char *
+build_cpp_args (const char *path,
+		const char *cpp_args)
+{
+	char *ret;
+	char *base;
+	int   i;
+
+	base = g_basename (path);
+	if (strlen (base) <= 4) {
+		ret = g_strconcat ("-D__ORBIT_IDL__ ", cpp_args, NULL);
+	} else {
+		/* base minus .idl extension */
+		base = g_strndup (base, strlen (base) - 4);
+		for (i = 0; base[i] != '\0'; i++) {
+			if (base[i] == '-')
+				base[i] = '_';
+		}
+		ret = g_strconcat ("-D__ORBIT_IDL__ -D__", base,
+				   "_COMPILATION ", cpp_args, NULL);
+		g_free (base);
+	}
+	return ret;
+}
+
 #define PARSE_FLAGS (IDLF_SHOW_CPP_ERRORS|	\
 		     IDLF_TYPECODES|		\
 		     IDLF_SRCFILES|		\
@@ -539,12 +564,15 @@ ORBit_iinterfaces_from_file (const char                     *path,
 			     CORBA_sequence_CORBA_TypeCode **typecodes_ret)
 {
 	ORBit_IInterfaces *retval;
+	char              *full_cpp_args;
 	IDL_tree           tree;
 	IDL_ns             namespace;
 	int                ret;
-
-	ret = IDL_parse_filename (path, cpp_args, NULL, &tree,
+	
+	full_cpp_args = build_cpp_args (path, cpp_args);
+	ret = IDL_parse_filename (path, full_cpp_args, NULL, &tree,
 				  &namespace, PARSE_FLAGS, 0);
+	g_free (full_cpp_args);
 	if (ret != IDL_SUCCESS) {
 		g_warning ("Cannot parse %s\n", path);
 		return NULL;
